#    this is CCCS.defn.tcl

#    slot ID for each IP
#    for correct operation of the PC (PCI) version the slots must be filled in the order A => F without gaps
set IPslots {A B C D E F}
set IPslot(IPDIO48) A
set IPslot(IPDAC) B
set IPslot(IP16ADC) C
set IPslot(IPQUADRATURE#1) D
set IPslot(IPQUADRATURE#2) E

#    what is expected to be in each slot - all IP are manufactured by SBS GreenSpring
set IPmanID(IPDIO48) 0xf0
set IPmodID(IPDIO48) 0x24

set IPmanID(IPDAC) 0xf0
set IPmodID(IPDAC) 0x16

set IPmanID(IP16ADC) 0xf0
set IPmodID(IP16ADC) 0x36

set IPmanID(IPQUADRATURE) 0xf0
set IPmodID(IPQUADRATURE) 0x41


#    options for the Digital48 

#    map between "logical" registers and "hardware" registers  [DIO(arm.limit) => register]
#    arm 5 (target angle) does not have limit switches

set DIO(0.limit) PADR_X_L
set DIO(1.limit) PADR_X_H
set DIO(2.limit) PBDR_X_L
set DIO(3.limit) PBDR_X_H
set DIO(4.limit) PADR_Y_L


set DIO(sense)   PADR_Y_H

#    magnetic switch sense to be looked for when positioning
#    DIO(arm.direction); direction = clockwise/anticlockwise

set DIO(1.C) 1
set DIO(1.A) 2
set DIO(2.C) 2
set DIO(2.A) 4
set DIO(3.C) 4
set DIO(3.A) 8
set DIO(4.C) 8
set DIO(4.A) 1


#    options for DAC setting

set currentDACchannel 1

#    map between control channels (arms) and DAC channels [DACchannels(arm) => DACchannel]
set DACchannels(0) 1
set DACchannels(1) 2
set DACchannels(2) 3
set DACchannels(3) 4
set DACchannels(4) 5
set DACchannels(5) 6


#   drive maximum  (range 1 => 100 equivalent to 1% => 100% of full DAC setting)
set DAC(c1.max) 100
set DAC(c2.max) 100
set DAC(c3.max) 100
set DAC(c4.max) 100
set DAC(c5.max) 50
set DAC(c6.max) 50

#    drive voltage (range 1 => 100 equivalent to 1% => 100% of drive maximum DAC setting)
set DAC(c1.high) 100
set DAC(c1.med)  50
set DAC(c1.low)  20
set DAC(c2.high) 100
set DAC(c2.med)  50
set DAC(c2.low)  20
set DAC(c3.high) 100
set DAC(c3.med)  50
set DAC(c3.low)  20
set DAC(c4.high) 100
set DAC(c4.med)  50
set DAC(c4.low)  20
set DAC(c5.high) 100
set DAC(c5.med)  50
set DAC(c5.low)  20
set DAC(c6.high) 100
set DAC(c6.med)  50
set DAC(c6.low)  20


#    drive polarity
#    for the target height clockwise = down and anticlockwise = up
set DAC(c1.clockwise)     1
set DAC(c1.anticlockwise) -1
set DAC(c2.clockwise)     1
set DAC(c2.anticlockwise) -1
set DAC(c3.clockwise)     -1
set DAC(c3.anticlockwise) 1
set DAC(c4.clockwise)     1
set DAC(c4.anticlockwise) -1
set DAC(c5.clockwise)     1
set DAC(c5.anticlockwise) -1
set DAC(c6.clockwise)     1
set DAC(c6.anticlockwise) -1


#    drive null values  (these can be obtained from the IP EPROM
set DAC(c1.idle) 2048
set DAC(c2.idle) 2048
set DAC(c3.idle) 2048
set DAC(c4.idle) 2048
set DAC(c5.idle) 2048
set DAC(c6.idle) 2048

#    drive period for "step" function  (millisecs)
set DAC(step_period) 1000

set StepsPerDegree 103.34

#    options for ADC setting

set currentADCchannel 1

#    map between control channels (arms) and ADC channels   [ADCchannels(arm) => ADCchannel]
set ADCchannels(0) 1
set ADCchannels(1) 2
set ADCchannels(2) 3
set ADCchannels(3) 4
set ADCchannels(4) 5
set ADCchannels(5) 6

#    adc zero values  (these can be obtained from the IP EPROM)
set ADC(c1.idle) 34
set ADC(c2.idle) 0
set ADC(c3.idle) 0
set ADC(c4.idle) 0
set ADC(c5.idle) 0
set ADC(c6.idle) 0

#    adc low threshold  (detects probable power switched off to motor)
set ADC(c1.low) 0
set ADC(c2.low) 0
set ADC(c3.low) 0
set ADC(c4.low) 0
set ADC(c5.low) 0
set ADC(c6.low) 0

#    adc high threshold (detects probable arm stall)
set ADC(c1.high) 1000
set ADC(c2.high) 1000
set ADC(c3.high) 1000
set ADC(c4.high) 1000
set ADC(c5.high) 1000
set ADC(c6.high) 1000

set ADC(monitor_period) 100


#    options for the Quadrature encoders

set currentQUADchannel 1

#    map between control channels (arms) and quadrature channels
#    quadrature channels 1-4 are on module QUADRATURE#1 and 
#    channels 5-8 are on module QUADRATURE#2
set QUADchannels(0) 1
set QUADchannels(1) 2
set QUADchannels(2) 3
set QUADchannels(3) 4
set QUADchannels(4) 5
set QUADchannels(5) 6
set QUADchannels(6) 7
set QUADchannels(7) 8

#    counter mode used by each control channel  (3 = quadrature; 0 = up/down; 4 = count/direction)
set QUADmode(0) 3
set QUADmode(1) 3
set QUADmode(2) 3
set QUADmode(3) 3
set QUADmode(4) 4
set QUADmode(5) 4
set QUADmode(6) 4
set QUADmode(7) 4

#    counter is required to appear to count positive as it moves away from zero point
#    Direction is + if hardware counter is already in this direction 
#    Direction is - if not
set CounterDirection(0) -
set CounterDirection(1) -
set CounterDirection(2) -
set CounterDirection(3) -
set CounterDirection(4) +
set CounterDirection(5) +
set CounterDirection(6) +
set CounterDirection(7) +




