#    Charissa Chamber Control System

set debug 0
set started 0

set frame cccs
set frametitle "Charissa Chamber Control System"
set icontitle "CCCS"

if {[info exists env(CCCS_Server)]} then {set resource $env(CCCS_Server)} else {set resource vme}

set testmode 0
set monitor 0

loadsource CCCS.defn

set motionsense 0

#    motion control   idle =0;  moving =1; calibrating =2/3; expert user movement = 4;  stalled =8;  limit =9
set motion_status 0
set motion_direction A
set motion_rate F

set movement_alerts 0
set motion_alerts 0

#    counter threshold - we slow down within this distance of required position
set motion_threshold 200

set motion_ID 0
set monitor_ID 0

#    arm current positions   (target height is arm 5 and target angle is arm 6)
set CurrentPosition(1) 0
set CurrentPosition(2) 0
set CurrentPosition(3) 0
set CurrentPosition(4) 0
set CurrentPosition(5) 0
set CurrentPosition(6) 0

#    arm required positions   (target height is arm 5 and target angle is arm 6)
set RequiredPosition(1) 0
set RequiredPosition(2) 0
set RequiredPosition(3) 0
set RequiredPosition(4) 0
set RequiredPosition(5) 0
set RequiredPosition(6) 0

#   monitor drive current option for each arm (0 = no; 1 = yes)
set MonitorCurrent(0) 1
set MonitorCurrent(1) 1
set MonitorCurrent(2) 1
set MonitorCurrent(3) 1
set MonitorCurrent(4) 1
set MonitorCurrent(5) 1

#   monitor limit switches option for each arm (0 = no; 1 = yes)
set MonitorLimits(0) 1
set MonitorLimits(1) 1
set MonitorLimits(2) 1
set MonitorLimits(3) 1
set MonitorLimits(4) 1
set MonitorLimits(5) 1

#   monitor movement option for each arm (0 = no; 1 = yes)
set MonitorMotion(0) 1
set MonitorMotion(1) 1
set MonitorMotion(2) 1
set MonitorMotion(3) 1
set MonitorMotion(4) 1
set MonitorMotion(5) 1

#   monitor calibration point option for each arm (0 = no; 1 = yes)
set MonitorCalPoint(0) 1
set MonitorCalPoint(1) 1
set MonitorCalPoint(2) 1
set MonitorCalPoint(3) 1
set MonitorCalPoint(4) 1
set MonitorCalPoint(5) 1


set arm_menu [list "arm 1" "arm 2" "arm 3" "arm 4" "target height" "target angle"]
set arm 0

#    list of menu items and associated actions for the EXPERT menu button 

set armlist {                                                                \
   {"move arm clockwise to right calibration point"               "move_to_calpoint C R"}      \
   {"move arm clockwise to left calibration point"                "move_to_calpoint C L"}      \
   {"move arm anticlockwise to right calibration point"           "move_to_calpoint A R"}      \
   {"move arm anticlockwise to left calibration point"            "move_to_calpoint A L"}      \
   {"step arm clockwise"                                    "start_motion S C"}      \
   {"step arm anticlockwise"                                "start_motion S A"}      \
   {"move arm clockwise"                                    "start_motion E C"}      \
   {"move arm anticlockwise"                                "start_motion E A"}      \
   {"update current position"                               "set_current_position"}  \
   {"define current position to be calibration point"       "set_calibration"}       \
}

set thlist {                                                                \
   {"step target down"                                    "start_motion S C"}      \
   {"step target up"                                      "start_motion S A"}      \
   {"move target down"                                    "start_motion E C"}      \
   {"move target up"                                      "start_motion E A"}      \
   {"update current position"                             "set_current_position"}  \
   {"define current position to be zero point"            "set_calibration"}       \
}

set talist {                                                                \
   {"step target clockwise"                                    "start_motion S C"}      \
   {"step target anticlockwise"                                "start_motion S A"}      \
   {"move target clockwise"                                    "start_motion E C"}      \
   {"move target anticlockwise"                                "start_motion E A"}      \
   {"update current position"                                  "set_current_position"}  \
   {"define current position to be zero point"                 "set_calibration"}       \
}

loadsource ipcommon


#    these file contain the IP register definitions
loadsource CCCS.DIGITAL48
loadsource CCCS.DAC
loadsource CCCS.ADC
loadsource CCCS.QUADRATURE


proc initialise {} {

     define-IPs
     setup-IPs
}

proc define-IPs {} {
    global IPmanID IPmodID IPslot

#    define all the IPs and check the configuration

#    IP-DIGITAL48
    define-DIGITAL48

    set manid [read-register ManID]
    set modid [read-register ModID]

    if {$manid != $IPmanID(IPDIO48) && $modid != $IPmodID(IPDIO48)} then {
        midas-warning "The IP in slot $IPslot(IPDIO48) is not the correct type.\nAn IP-Digital48 module is required."
    } else {
        log "IP Digital 48 in slot $IPslot(IPDIO48) OK"
    }

#    IP-DAC
    define-DAC

    set manid [read-register ManID]
    set modid [read-register ModID]

    if {$manid != $IPmanID(IPDAC) && $modid != $IPmodID(IPDAC)} then {
        midas-warning "The IP in slot $IPslot(IPDAC) is not the correct type.\nAn IP-DAC module is required."
    } else {
        log "IP DADC in slot $IPslot(IPDAC) OK"
    }

#    IP-16ADC
    define-ADC

    set manid [read-register ManID]
    set modid [read-register ModID]

    if {$manid != $IPmanID(IP16ADC) && $modid != $IPmodID(IP16ADC)} then {
        midas-warning "The IP in slot $IPslot(IP16ADC) is not the correct type.\nAn IP-16ADC module is required."
    } else {
        log "IP ADC in slot $IPslot(IP16ADC) OK"
    }

#    IP-QUADRATURE
    define-QUADRATURE 1

    set manid [read-register ManID]
    set modid [read-register ModID]

    if {$manid != $IPmanID(IPQUADRATURE) && $modid != $IPmodID(IPQUADRATURE)} then {
        midas-warning "The IP in slot $IPslot(IPQUADRATURE#1) is not the correct type.\nAn IP-QUADRATURE module is required."
    } else {
        log "IP QUADRATURE in slot $IPslot(IPQUADRATURE#1) OK"
    }

    define-QUADRATURE 2

    set manid [read-register ManID]
    set modid [read-register ModID]

    if {$manid != $IPmanID(IPQUADRATURE) && $modid != $IPmodID(IPQUADRATURE)} then {
        midas-warning "The IP in slot $IPslot(IPQUADRATURE#2) is not the correct type.\nAn IP-QUADRATURE module is required."
    } else {
        log "IP QUADRATURE in slot $IPslot(IPQUADRATURE#2) OK"
    }

}

proc setup-IPs {} {
    global mname DAC currentDACchannel currentADCchannel QUADmode

#    perform any initial setup required

#    IP-DIO48

     set mname IPDIO48

#    set all unused port pins as outputs (=1)

    write-register PADDR_X 0
    write-register PADDR_Y 0

    write-register PBDDR_X 0
    write-register PBDDR_Y 255

    write-register PCDDR_X 255
    write-register PCDDR_Y 255

#    set all data bits => 0

    write-register PADR_X 0
    write-register PADR_Y 0

    write-register PBDR_X 0
    write-register PBDR_Y 0

    write-register PCDR_X 0
    write-register PCDR_Y 0

#    set for simple bit i/o    mode 0 submode 1X  (unidirectional 8 bit)

    write-register PGCR_X 0
    write-register PGCR_Y 0

    write-register PACR_X 0x80
    write-register PBCR_X 0x80

    write-register PACR_Y 0x80
    write-register PBCR_Y 0x80


#    IP-DAC
#    the DAC outputs -10V to +10V and so initialise all DAC channels to 0V

     set mname IPDAC
     set currentDACchannel 1
     for {set i 1} {$i <= 6} {incr i} {write-register c$i.DAC $DAC(c$i.idle)}

#    IP-16ADC
#    reset and setup the ADC; the ADC range is set to -5 to +5 V

     set mname IP16ADC
     set currentADCchannel 1
     setup_adc

#    IP-QUADRATURE
#     there are 4 channels per module and we have 6 items to move => 2 modules

     set mname IPQUADRATURE#1

#    enable TTL terminators
      write-QUAD-CSR 3

     for {set c 1} {$c <= 4} {incr c} {

        set i [expr $c - 1]

#    perform master reset
         write-QUAD-CR $c 0x20
 
#    Z function is disabled
         write-QUAD-CHCR $c 0

#    set counting mode
         write-QUAD-ICR $c [expr ($QUADmode($i) >> 2) & 1]

#    enable quadrature
         write-QUAD-QR $c [expr $QUADmode($i) & 3]

#    
         write-QUAD-OCR $c 0x10

#    enable counting
         write-QUAD-ICR $c [expr (($QUADmode($i) >> 2) & 1) + 8]
     }

     set mname IPQUADRATURE#2

#    enable TTL terminators
      write-QUAD-CSR 3

     for {set c 1} {$c <= 4} {incr c} {

        set i [expr $c - 1 + 4]

#    perform master reset
         write-QUAD-CR $c 0x20
 
#    Z function is disabled
         write-QUAD-CHCR $c 0

#    set counting mode
         write-QUAD-ICR $c [expr ($QUADmode($i) >> 2) & 1]

#    enable quadrature
         write-QUAD-QR $c [expr $QUADmode($i) & 3]

#    
         write-QUAD-OCR $c 0x10

#    enable counting
         write-QUAD-ICR $c [expr (($QUADmode($i) >> 2) & 1) + 8]
     }

}


proc click-arm {v} {
    global arm motion_status
    global DACchannels currentDACchannel ADCchannels currentADCchannel QUADchannels currentQUADchannel
    global RequiredPosition CurrentPosition
    global MonitorCurrent MonitorLimits MonitorMotion MonitorCalPoint
    global armlist thlist talist
    global StepsPerDegree

    if {$motion_status != 0} then {
        midas-warning "Please STOP the current arm movement first"
        return
    }

    set arm $v
    set id [expr $arm + 1]

#    convert the selected arm number into hardware DAC and ADC channels
    set currentDACchannel $DACchannels($v)
    set currentADCchannel $ADCchannels($v)
    set currentQUADchannel $QUADchannels($v)

    set CurrentPosition($id) [read-counter]

    :modify ARM :value $v
    :modify REQUIRED1 :value $RequiredPosition($id)
    :modify REQUIRED2 :value [string range [expr $RequiredPosition($id) / $StepsPerDegree] 0 5]
    :modify CURRENT1  :inactive F :value $CurrentPosition($id)  :inactive T
    :modify CURRENT2  :inactive F :value [string range [expr $CurrentPosition($id) / $StepsPerDegree] 0 5]  :inactive T

    :modify MONC  :value $MonitorCurrent($arm)
    :modify MONL  :value $MonitorLimits($arm)
    :modify MONM  :value $MonitorMotion($arm)
    :modify MONP  :value $MonitorCalPoint($arm)

    if {$arm > 3} then {
        :modify CALIBRATE1 :inactive T
        :modify CALIBRATE2 :inactive T
    } else {
        :modify CALIBRATE1 :inactive F
        :modify CALIBRATE2 :inactive F
    }

#    set appropiate expert list of actions

    set menu {}
    if {$arm <= 3} then {
      foreach item $armlist {lappend menu [first $item]}
      :modify EXPERT  :strings $menu
    }
    if {$arm == 4} then {
      foreach item $thlist {lappend menu [first $item]}
      :modify EXPERT  :strings $menu
    }
    if {$arm == 5} then {
      foreach item $talist {lappend menu [first $item]}
      :modify EXPERT  :strings $menu
    }
}

proc select_adc_channel {} {
    global mname currentADCchannel

    set mname IP16ADC

    set CSR [expr ([read-register Status] & 0xffe0) + 0x18 + ($currentADCchannel - 1)]
    write-register Control $CSR
}

#    the ADC range is -5 to +5 V
proc setup_adc {} {
    global mname currentADCchannel

    set mname IP16ADC

    set CSR [expr ([read-register Status] & 0xffe0) + 0x18 + ($currentADCchannel - 1)]
    set CSR [expr ($CSR & 0xff9f) + (3 << 5)]
    set CSR [expr ($CSR & 0xdfff) + 0x2000]
    write-register Control $CSR
}

#    read the channel counter register
proc read-counter {} {
    global mname currentQUADchannel
    global arm CounterDirection

    if {$currentQUADchannel <= 4} then {
        set mname IPQUADRATURE#1
        set c $currentQUADchannel
    } else {
        set mname IPQUADRATURE#2
        set c [expr $currentQUADchannel - 4]
    }

#    write bits [1..0] of the counter control register to 1 bits. 
#    this resets the byte sequence pointer and transfers the counter contents to the Output Latch
     write-QUAD-CR $c 0x03

#    read the 3 bytes of the Output Latch
     set b0 [read-register Ch$c.Counter]
     set b1 [read-register Ch$c.Counter]
     set b2 [read-register Ch$c.Counter]

     return [expr 0 $CounterDirection($arm) (((($b2 << 16) + ($b1 << 8) + $b0) << 8) >> 8)]
}

proc write-counter {v} {
    global mname currentQUADchannel
    global arm CounterDirection


    if {$currentQUADchannel <= 4} then {
        set mname IPQUADRATURE#1
        set c $currentQUADchannel
    } else {
        set mname IPQUADRATURE#2
        set c [expr $currentQUADchannel - 4]
    }

    if {$arm > 3 && $v == 0} then {set v 1}

     set vl [expr 0 $CounterDirection($arm) $v]

     set b0 [expr $vl & 255]
     set b1 [expr ($vl >> 8) & 255]
     set b2 [expr ($vl >> 16) & 255]

#    set bit 0 of the counter control register to 1
#    this resets the byte sequence pointer
     write-QUAD-CR $c 0x01

#    write the 3 bytes of the PCR
     write-register Ch$c.PCR $b0
     write-register Ch$c.PCR $b1
     write-register Ch$c.PCR $b2

#    set bit 3 of the counter control register to 1
#    this transfers the contents of the PCR to the counter
     write-QUAD-CR $c 0x08
}

proc zero-counter {} {write-counter 0}

proc reset-counter {} {
    global mname currentQUADchannel

    if {$currentQUADchannel <= 4} then {
        set mname IPQUADRATURE#1
        set c $currentQUADchannel
    } else {
        set mname IPQUADRATURE#2
        set c [expr $currentQUADchannel - 4]
    }

#    set bit 5 of the counter control register to 1
#    this resets the Counter
     write-QUAD-CR $c 0x20
} 


proc click-required {v} {
    global arm RequiredPosition motion_status
    global StepsPerDegree

    if {$motion_status != 0} then {
        midas-warning "Please STOP the current arm movement first"
        return
    }

    set id [expr $arm + 1]
    set RequiredPosition($id) $v

    :modify REQUIRED1 :value $RequiredPosition($id)
    :modify REQUIRED2 :value [string range [expr $RequiredPosition($id) / $StepsPerDegree] 0 5]
}


proc click-current {v} {
    global motion_status

    if {$motion_status != 0} then {
        midas-warning "Please STOP the current arm movement first"
        return
    }

    write-counter $v
    set_current_position
}

proc move_to_target {args} {
    global arm arm_menu motion_status
    global RequiredPosition CurrentPosition

    if {$motion_status != 0} then {
        midas-warning "Please STOP the current arm movement first"
        return
    }

    set data [read-counter]

    set id [expr $arm + 1]
    if {$data != $CurrentPosition($id)} then {

#    counter has changed !!!!

        warning "Current position of [lindex $arm_menu $arm] should be $CurrentPosition($id) but the counter reads $data - corrected"
   #     write-counter $CurrentPosition($id)
    }

    set_current_position

    if {[expr $RequiredPosition($id) - $CurrentPosition($id)] == 0} then {
       :modify STATUS :label "[lindex $arm_menu $arm] is already at the required position"
       return
    }

    log "Moving [lindex $arm_menu $arm] to position $RequiredPosition($id)"

    if {[expr $RequiredPosition($id) - $CurrentPosition($id)] > 0} then {start_motion M A} else {start_motion M C}

    if {$motion_status != 0} then {
       :modify MOVE1 :inactive T
       :modify MOVE2 :inactive T
    }
}

proc move_to_calpoint {D W} {
    global DIO calsense calpoint
    global arm arm_menu
    global motion_status
#
#    the current arm is moved to a calibration point
#
#    D=C (clockwise) D=A (anticlockwise); W=R (right) W=L (left)

    if {$motion_status != 0} then {
        midas-warning "Please STOP the current arm movement first"
        return
    }

#    this determines the value to be read from the calibration point magswitch sensor
    switch $W {
        R  {set calsense $DIO([expr $arm + 1].$D); set calpoint "right"}
        L  {set calsense $DIO([expr $arm + 1].$D); set calpoint "left"}
    }

    log "Moving [lindex $arm_menu $arm] to $calpoint calibration point"

    start_motion C $D

    if {$motion_status != 0} then {
        :modify MOVE1 :inactive T
        :modify MOVE2 :inactive T
    }
}

proc start_motion {M D} {
#
#    the current arm is moved
#
#         M=S (step) M=M (move) M=C (calibrate) M=E (expert user);  D=C (clockwise) D=A (anticlockwise)

     global mname DAC DIO currentDACchannel
     global arm arm_menu
     global motion_status motion_direction motion_rate motion_ID motion_alerts movement_alerts
     global calpoint motionsense

     set motion_direction $D

     if {$arm <= 3} then {

#    set sense of magnetic switches to stop on

         switch $D {
              C    {set motionsense $DIO([expr $arm + 1].C)}
              A    {set motionsense $DIO([expr $arm + 1].A)}
         }
     }

     set mname IPDAC

#    basic DAC setting

     set value [expr int(2048 * ($DAC(c$currentDACchannel.max) / 100.0))]
  
#    move/calibrate at high speed and step at medium speed
     switch $M {
         S   {set value [expr int($value * ($DAC(c$currentDACchannel.med) / 100.0))]}
         M   {set value [expr int($value * ($DAC(c$currentDACchannel.high) / 100.0))]}
         C   {set value [expr int($value * ($DAC(c$currentDACchannel.high) / 100.0))]}
     }

#    apply polarity
     switch $D {
         C   {
                 set value [expr $value * $DAC(c$currentDACchannel.clockwise)]
                 if {$arm != 4} then {set direction "clockwise"} else {set direction "down"}
             }
         A   {
                 set value [expr $value * $DAC(c$currentDACchannel.anticlockwise)]
                 if {$arm != 4} then {set direction "anticlockwise"} else {set direction "up"}
             }
     }

#    apply 0V offset
     set value [expr $DAC(c$currentDACchannel.idle) + $value]

#    safety checks
     if {$value > 4095} then {set value 4095}
     if {$value < 0}    then {set value 0}

#    write drive Voltage 
     write-register c$currentDACchannel.DAC $value
     :modify DAC :value [expr $value - $DAC(c$currentDACchannel.idle)]

#   now moving

     switch $M {
         S   {:modify STATUS :label "Stepping [lindex $arm_menu $arm]"}
         E -
         M   {:modify STATUS :label "Moving [lindex $arm_menu $arm] $direction"}
         C   {:modify STATUS :label "Moving [lindex $arm_menu $arm] $direction to the $calpoint calibration point"}
     }

#    if stepping stop after n millisecs
     if {$M == "S"} then {set motion_ID [after $DAC(step_period) do_stop]}

#    set status to indicate moving and start monitoring current to drive motor

     switch $M {
         S   {set motion_status 1}
         M   {set motion_status 1}
         C   {set motion_status 2}
         E   {set motion_status 4}
     }

     set motion_rate F
     set motion_alerts 0
     set movement_alerts 0
     monitor_motion
}

proc do_stop {args} {
     global arm arm_menu

     stop_motion

     :modify STATUS :label "[lindex $arm_menu $arm] is now idle"
     :modify ADC :value 0
}

#    this establishes the position of the current arm
proc do_calibrate {w1 w2} {
    global motion_status

    if {$motion_status != 0} then {
        midas-warning "Please STOP the current arm movement first"
        return
    }

    move_to_calpoint $w1 $w2
    if {$motion_status != 0} then {set motion_status 3}
}

proc stop_motion {} {
     global mname DAC ADC currentDACchannel
     global arm arm_menu RequiredPosition CurrentPosition
     global motion_status motion_ID

     :modify MOVE1 :inactive F
     :modify MOVE2 :inactive F

     after cancel $motion_ID
     set motion_ID 0

     set mname IPDAC
     write-register c$currentDACchannel.DAC $DAC(c$currentDACchannel.idle)
     :modify DAC :value 0

     :modify ADC :value 0

#    update current arm position

     set_current_position

     set id [expr $arm + 1]
     log "[lindex $arm_menu $arm] stopped at position $CurrentPosition($id)"

     set motion_status 0

#    and schedule an update again to allow for arm inertia
     after 500 set_current_position
}


proc monitor_motion {} {
    global EGreport
    global mname ADC currentADCchannel DAC currentDACchannel DIO
    global motion_status motion_direction motion_rate monitor_ID  motion_ID movement_alerts
    global arm arm_menu
    global calsense calpoint motionsense MonitorCalPoint
    global RequiredPosition CurrentPosition
    global motion_threshold

    if {$motion_status == 0} then {return}

#    first monitor drive motor current for overload

    set save $EGreport
    set EGreport -1

    if {[monitor_adc] != 0} then {

#    fault detected - stop arm motion
    
         stop_motion
    }

    set EGreport $save

    if {$motion_status == 0} then {return}

    set save $EGreport
    set EGreport -1

    set position_state [monitor_position]
    if {$position_state == 2} then {

#    fault detected - stop arm motion

         stop_motion
    }

    set EGreport $save

    if {$motion_status == 0} then {return}

#    next monitor limit switches

    set save $EGreport
    set EGreport -1

    if {[monitor_limits] != 0} then {

#    fault detected - stop arm motion

         stop_motion
    }

    set EGreport $save

    if {$motion_status == 0} then {return}

    if {$motion_status == 2 || $motion_status == 3} then {

#    calibrating - check if the calibration point has been reached

       set save $EGreport
       set EGreport -1

       set mname IPDIO48
       set data [expr [read-register $DIO(sense)]]
       if {[expr $data & $calsense] == $calsense} then {
           :modify STATUS :label "[lindex $arm_menu $arm] halted at the $calpoint calibration sense switch"
           if {$motion_status == 3} then {after 500 set_calibration}
           stop_motion
       }
       set EGreport $save
    }

    if {$motion_status == 0} then {return}

    if {$motion_status == 1 && $arm <= 3} then {

#    check if we have reached a calibration point  (arms 1 => 4 only)

        set save $EGreport
        set EGreport -1

        set mname IPDIO48
        set data [expr [read-register $DIO(sense)]]
        if {[expr $data & $motionsense] == $motionsense} then {
           if {$MonitorCalPoint($arm) == 1} then {
              if {$motion_direction == "C"} then { 
                 :modify STATUS :label "[lindex $arm_menu $arm] halted at the left calibration sense switch"
              } else {
                 :modify STATUS :label "[lindex $arm_menu $arm] halted at the right calibration sense switch"
              }
              stop_motion
           } else {
              if {$motion_direction == "C"} then { 
                 :modify STATUS :label "[lindex $arm_menu $arm] passing the left calibration sense switch"
              } else {
                 :modify STATUS :label "[lindex $arm_menu $arm] passing the right calibration sense switch"
              }
           }
        }
        set EGreport $save
    }

    if {$motion_status == 0} then {return}

    if {$motion_status == 1} then {

#    moving - check current position

        set save $EGreport
        set EGreport -1

        if {$position_state != 0} then {

#    reached target  - stop arm motion
       
             stop_motion
             :modify STATUS :label "[lindex $arm_menu $arm] halted at the required position"
        } else {

#       look how far left to move and possibly reduce drive voltage

            set id [expr $arm + 1]
            set remaining [expr $RequiredPosition($id) - $CurrentPosition($id)]
            if {[expr abs($remaining)] < $motion_threshold} then {
            if {$motion_rate == "F"} then {

#    calculate new drive voltage

               set value [expr int(2048 * ($DAC(c$currentDACchannel.max) / 100.0))]
               set value [expr int($value * ($DAC(c$currentDACchannel.low) / 100.0))]

               switch $motion_direction {
                   C   {set value [expr $value * $DAC(c$currentDACchannel.clockwise)]}
                   A   {set value [expr $value * $DAC(c$currentDACchannel.anticlockwise)]}
               }
               set value [expr $DAC(c$currentDACchannel.idle) + $value]
               if {$value > 4095} then {set value 4095}
               if {$value < 0}    then {set value 0}

#    write drive Voltage 
               set mname IPDAC
               write-register c$currentDACchannel.DAC $value
               :modify DAC :value [expr $value - $DAC(c$currentDACchannel.idle)]

               set motion_rate L
               log "[lindex $arm_menu $arm] slowing down at position $CurrentPosition($id)" 
            } }
        }
        set EGreport $save
    }

    if {$motion_status == 0} then {return}

    set monitor_ID [after $ADC(monitor_period) monitor_motion]
}


proc monitor_adc {} {
    global mname ADC currentADCchannel DAC currentDACchannel
    global motion_alerts
    global arm_menu arm
    global MonitorCurrent

    set reply 0

    set mname IP16ADC

#    force adc to convert
    write-register Convert -1

#    and read it
    set adc [expr abs([read-register Data] - 32768 - $ADC(c$currentADCchannel.idle))]
    :modify ADC :value $adc

#    check for out of range

    if {$adc > $ADC(c$currentDACchannel.high)} then {
        incr motion_alerts
    } else {set motion_alerts 0}

    if {$motion_alerts > 10} then {

#    arm has stalled

        if {$MonitorCurrent($arm) != 0} then {
            alarm "[date] [lindex $arm_menu $arm] has stalled - check for obstruction to motion"
            :modify STATUS :label "[lindex $arm_menu $arm] has stalled"
            set reply 1
        } else {
           warning "[date] [lindex $arm_menu $arm] has exceeded drive current high threshold"
           set motion_alerts 0
        }
    }
   return $reply
}

proc monitor_limits {} {
    global mname arm arm_menu DIO
    global MonitorLimits

#    there are no limit switches for the target angle - control item 5
    if {$arm == 5} then {return 0}

    set reply 0

    set mname IPDIO48

#    read arm limit switches

    set data [read-register $DIO($arm.limit)]
    if {$data == 5} then {return 0}

#    one (or both) limit switches not open

    if {[expr $data >> 2] != 1} then {
            if {$MonitorLimits($arm) != 0} then {
                alarm "[date] [lindex $arm_menu $arm] has tripped left/upper limit switch - check for obstruction to motion"
                :modify STATUS :label "[lindex $arm_menu $arm] halted at limit"
                set reply 1
            } else {
                warning "[date] [lindex $arm_menu $arm] has tripped left/upper limit switch"
            }
        return $reply
    }
    if {[expr $data & 3] != 1} then {
           if {$MonitorLimits($arm) != 0} then {
               alarm "[date] [lindex $arm_menu $arm] has tripped right/lower limit switch - check for obstruction to motion"
               :modify STATUS :label "[lindex $arm_menu $arm] halted at limit"
               set reply 1
           } else {
                warning "[date] [lindex $arm_menu $arm] has tripped right/lower limit switch"
           }
        return $reply
    }

#    switches not in obvious state 
    if {$MonitorLimits($arm) != 0} then {
               warning "[lindex $arm_menu $arm] limit switches returned $data - unknown state"
               :modify STATUS :label "[lindex $arm_menu $arm] stopped"
               set reply 1
    }
    return $reply
}

proc monitor_position {} {
   global arm arm_menu motion_direction RequiredPosition CurrentPosition
   global movement_alerts
   global MonitorMotion StepsPerDegree

#    read current counter 

   set current [read-counter]
   :modify CURRENT1  :inactive F :value $current  :inactive T
   :modify CURRENT2  :inactive F :value [string range [expr $current / $StepsPerDegree] 0 5]  :inactive T

   set id [expr $arm + 1]

    if {$CurrentPosition($id) == $current} then {
        incr movement_alerts
    } else {set movement_alerts 0}

    if {$movement_alerts > 20} then {

#    arm is not moving

        if {$MonitorMotion($arm) != 0} then {
           alarm "[date] [lindex $arm_menu $arm] is not moving - check motor drive is switched on!"
           :modify STATUS :label "[lindex $arm_menu $arm] is not moving"
           return 2
        } else {
           warning "[date] [lindex $arm_menu $arm] is not moving"
           set movement_alerts 0
        }
    }

   set required $RequiredPosition($id)
   set CurrentPosition($id) $current

   switch $motion_direction {
       C     {if {[expr $required - $current] >= 0} then {return 1}}
       A     {if {[expr $required - $current] <= 0} then {return 1}}
   }

   return 0
}

proc set_calibration {} {

#    zero the counter 
    zero-counter
#    establish this as the current position
    set_current_position
}

proc set_current_position {} {
   global arm CurrentPosition StepsPerDegree

#    establish the current position
   set data [read-counter]
   :modify CURRENT1  :inactive F :value $data :inactive T
   :modify CURRENT2  :inactive F :value [string range [expr $data / $StepsPerDegree] 0 5] :inactive T

    set id [expr $arm + 1]
    set CurrentPosition($id) $data
}


proc click arg {
   global debug actionlist
   global MonitorCurrent MonitorLimits MonitorMotion MonitorCalPoint arm
   global armlist thlist talist
   global StepsPerDegree

   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      ARM      {click-arm $v}
      REQUIRED1 {click-required $v}
      REQUIRED2 {click-required [expr int ($v * $StepsPerDegree)]}
      CURRENT1  {click-current $v}
      CURRENT2  {click-current [expr int ($v * $StepsPerDegree)]}

      MONC     {set MonitorCurrent($arm) $v}
      MONL     {set MonitorLimits($arm) $v}
      MONM     {set MonitorMotion($arm) $v}
      MONP     {set MonitorCalPoint($arm) $v}

      EXPERT  {
           switch $arm {
             0 -
             1 -
             2 -
             3   {set action [lindex $armlist $i]}
             4   {set action [lindex $thlist $i]}
             5   {set action [lindex $talist $i]}
           }
           eval [second $action]
      }

      DIAGNOSTICS {
          switch $v {
            IP-Digital48    {inform ipdio48 "Start 0"}
            IP-DAC          {inform ipdac   "Start 1"}
            IP-16ADC        {inform ip16adc "Start 2"}
            IP-Quadrature1  {inform ipquadrature "Start 3"}
            IP-Quadrature2  {inform ipquadrature "Start 4"}
          }
      }

      default   {puts stdout $arg}
   }
}

proc click-save {args} {
   global env CurrentPosition

    set savefile [file join $env(EXPERIMENTDIR) SaveFile]

    set z [catch {open $savefile w} file]
    if {$z != 0} then {set-footer "$file"; return}

    for {set c 1} {$c <= 6} {incr c} {puts $file $CurrentPosition($c)}

    close $file
}

proc click-restore {args} {
   global env arm CurrentPosition

    set savefile [file join $env(EXPERIMENTDIR) SaveFile]

    set z [catch {open $savefile r} file]
    if {$z != 0} then {set-footer "$file"; return}

    set currentarm $arm

    for {set c 1} {$c <= 6} {incr c} {
        click-arm [expr $c - 1]
        set CurrentPosition($c) [gets $file]
        set-footer "restoring position of arm $arm as $CurrentPosition($c)"
        write-counter $CurrentPosition($c)
    }

    close $file

    click-arm $currentarm
}

proc make-frame {} {
  global env 
  global arm arm_menu

set rows 10
set cols 90

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/charissa" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 20

     :choice-stack ARM :label "Select item to move" :strings $arm_menu :value $arm
     :hskip 100
     :message DATETIME :label "[date]" :bold T

   :panel CLICK :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 20

       :message M1 :label "Steps" :bold T
       :next-row T
       :text CURRENT1    :label "Current Position"  :w 6 :inactive T
       :slider REQUIRED1 :label "Required Position" :min -25000 :max 25000
       :button MOVE1     :label "Move"    :command "move_to_target"
       :button STOP1      :label "Stop"   :command "do_stop" :colour red

       :next-row T

       :message M12 :label "Degrees" :bold T
       :next-row T
       :text CURRENT2    :label "Current Position"  :w 6 :inactive T
       :slider REQUIRED2 :label "Required Position" :min -250 :max 250
       :hskip 25
       :button MOVE2     :label "Move"    :command "move_to_target"
       :button STOP2      :label "Stop"   :command "do_stop" :colour red

      :row-gap 10
      :next-row T
      :checkbox MONC  :label "Monitor drive current" 
      :checkbox MONL  :label "Monitor limits" 
      :checkbox MONM  :label "Monitor Movement" 
      :checkbox MONP  :label "Monitor Calibration point"

       :next-row T
       :button ZERO :label "Define current position to be zero point" :command "set_calibration"

       :next-row T
       :button CALIBRATE1 :label "Move clockwise to next calibration point"  :command "do_calibrate C L"
       :button CALIBRATE2 :label "Move anti-clockwise to next calibration point"  :command "do_calibrate A R"

   :panel P1 :layout V :padx 5 :pady 5
     [:path P1] configure -relief raised -bd 1

     :message STATUS :label "" :bold T

   :panel MONITOR :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 20

      :slider DAC :label "Drive Voltage" :min -2048 :max +2048 
      :slider ADC :label "Drive Current" :min 0 :max 2000 

   :panel LOG :layout V
    :galley LIST :rows $rows :cols $cols :deaf T  :font variable

    :modify LIST :createtag "alarm  -foreground red"
    :modify LIST :createtag "log -foreground white"
    :modify LIST :createtag "warning -foreground yellow"
    :modify LIST :createtag "footer -foreground white"
   
   :panel BUTTONS  :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 10

      :button CLEAR         :label "Empty Text Window"           :command ":modify LIST :empty T"
      :button INITIALISE    :label "Initialise Control System"   :command "initialise"
      :button SAVE          :label "Save"                        :command "click-save"
      :button RESTORE       :label "Restore"                     :command "click-restore"
      :menubutton EXPERT      :strings ""
      :menubutton DIAGNOSTICS :strings [list IP-Digital48 IP-DAC IP-16ADC IP-Quadrature1 IP-Quadrature2]

   :show T
}

proc update_datetime {} {
    :modify DATETIME :label "[date]"
    after 1000 update_datetime
}

proc start args {
   global started
   if {$started == 0} then {
      make-frame
      click-arm 0
      set started 1
      after 1000 update_datetime
   } else {open-frame}
}

proc set-footer {s} {:modify LIST :usingtag footer  :strings "$s\n"}
proc log {s}        {:modify LIST :usingtag log     :strings "[date] $s\n"}
proc alarm {s}      {:modify LIST :usingtag alarm   :strings "[date] $s\n"}
proc warning {s}    {:modify LIST :usingtag warning :strings "[date] $s\n"}

